/* 
This is the program file for "Hypothesis Testing with Near-Unit Roots:  The  
Case of Long-Run Purchasing-Power Parity," by Michael Dueker (July/August 1993).
This particular program gives results for the nominal exchange rate
between Germany/Japan.  Starting values are in bgm, which estimates an
ARFIMA(2,d,1) model of the nominal exchange rate.  bgm[1]= 1 -order of
integration; bgm[2:3] represents the AR parameter; bgm[4] equals the MA 
parameter; and, bgm[5]= ln (sigma^2).  In the proc hu, pe represents the 
nominal exchange rate, per represents the real exchange rate, and pec 
represents the price level ratio.  pe* is the vector of periodogram 
ordinates, and spd is the vector of ordinates for spectral density.

NOTE:  This program can be used for any of the other countries in the study
       by simply uncommenting the variables for that country and commenting 
       out the variables for Germany/Japan!!  (See the section of the 
       program entitled "Step 3:  Reading in Data."
*/

@ GMMQ.S24 @    @ This file is for Version 2 of GAUSS @

library optmum;
#include optmum.ext;
optset;

@------------------------------------------------------------------------------
Prepared by Lars P. Hansen, John C. Heaton, and Masao Ogaki
    Financial support from the National Science Foundation
                 Grant number: SES-8512371

 Last Revision:  03/06/89
------------------------------------------------------------------------------@
/*
This program has been used and seems to be free of errors. However,
we (Lars P. Hansen, John C. Heaton, and Masao Ogaki) do not assume
responsibility for any remaining errors.  In no event shall we be liable to
for any damages whatsoever arising out of the use of or inability to use
this program.

This program is for the generalized method of moments (GMM) procedure.
See "Generalized Instrumental Variable Estimation of Nonlinear Rational
Expectations Models," by L.P. Hansen and K.J. Singleton (Econometrica
1982, Vol.50, 1269-1286).  This example file is for Hansen and Singleton
model, but uses different data than theirs.

   @@ To run this example file, type
       RUN MINQUAD.SET [F4][F2]
       RUN GMMQ.SET [F4][F2]
       RUN GMMQ.EXP [F4][F2]

      The programs MINQUAD.SET and GMMQ.SET define necessary procedures.
      To use these programs for the user's problem, the user can
      modify this example file for the user's problem.
      Usually the user does not have to modify MINQUAD.SET and GMMQ.SET
      because all the paramters for these two programs are controlled
      by this example file.

   @@ The user should go through Step 1-4 and modify this example file for
      the user's own problem.

Model:   E[w(t)|I(t)]=0:  Disturbance w(t) nw by 1 vector
            wi(t) is in information set I(t+ri) i=1,2,..,nw
         h(t,b)=[z(t)w(t)]:  Instruments z(t) L by nw matrix
            z(t) is in I(t).
            It is assumed that z(t) of the form
              z(t)=|z1(t) 0   ....    0|
                   | 0 z2(t) 0 ....   0|
                   | |                 |
                   | 0  ....   0 znw(t)|,    so that

              [z(t)w(t)]'=[w1(t)z1(t)',...,wnw(t)znw(t)'].

Algorithm:
   This program calculates and iterates the following:
     ititial W0 given.

  1. g(b)=C*[h(1,b)+,...,+h(T,b)]
     b=argmin{g(t,b)'W0*g(t,b)}
     z(t)w(t)=h(t,b)

  2. Rzw(j)=(1/T)[z(1+j)w(1+j)w(1)'z(1)'+,..,+{z(T)w(T)w(T-j)Z(T-j)]
                       j=0,1,..,rm
     W1=Rzw(0)+{Rzw(1)+Rzw(1)'}+,..,+{Rzw(r)+Rzw(r)'}
     W0=inv(W1)
     (W1 can be calculated by other methods depending on the value of
      calwflag.  See below for more explanations.)

  3. go back to 1.

   Here C is a scaling multiplier, and taken to be 'const' or 'const2'
   depending on whether W0 is I or not, and r=maxc(rwv).  See below for
   more explanations.

   @@ The following values in GAUSS matrix files:
        a. Current parameter values in MINQUAD.SET search: crparv.fmt
        b. Current inverse hessian matrix in MINQUAD.SET search: crhess.fmt
        c. GMM estimates which uses W0=I: bgmi.fmt
        d. GMM estimates of the last GMM iteration: bgm.fmt
        e. W0 calculated from the last GMM iteration results: w0.fmt

*/
@ ======================== User Definition Area ============================ @
@ ********************************************************************** @
@ ----------- Step 1: Prepare Output File ---------------------------@
@ ********************************************************************** @
output file=c:\letter\fcoin5.s24 reset; @ Specify the name of the output file@

?"
************************** GMM Results *********************************";

    @ Prepare the following message which will be printed
      at the top of the output file @
?"
fcoin5.s24 from gmmq.s24";
@ ? "Uses Hansen-Ogaki program"; @

@ *********************************************************************** @
@------------ Step 2:  Define Global Variables  ------------------------- @
@ *********************************************************************** @

@----------------------------@
   @ Section A. User must specify variables in this section. @
@----------------------------@

                 @ # of observations=T; scalar @
tend= 239; tend=233;
bgm= .17|-.379|-.2|.230|-2;
@ bgm= -.6|-.79|.23|-3.2; @
hsec;


                      @ bgm=b(1)|..|b(kgm); kgm by 1 vector
                      Initial values of the coefficients
                 (In this example, the parameters happen to converge to
                  values that are not economically meaningful when W0=I.
                  The initial values are set to values close to the converged
                  values in this example, so that this example converges
                  quickly.) @

nw=5;            @ # of disturbance terms in w(t); scalar @

nzv=ones(5,1);            @ nzv=nz(1)|..|nz(nw); nw by 1 vector
                   For each i, nz(i) is # of elements in zi(t) i=1,..,nw.
                   Let L=sumc(nzv), then L is # of orthogonal condition.@
             @ no. of columns = nw i.e. no. of orthog. for each disturb@

rwv=ones(5,1);           @ rwv=rw(1)|..|rw(nw); nw by 1 vector
                    rw(i) is defined so that
                     wi(t) is in I(t+rw(i)) i=1,..,nw @

@---------------------------------@
     @ Section B.  User can leave the variables in this section as they are,
                   and go to STEP 3.

                   Only advanced users should modify this section.@

@---------------------------------@

gradname=&GRAD2; @ Specify the name of proc that calculates
                    the gradient dgT(b)/db. @

const = .1/sqrt(tend);    @ Scaling Multiplier when W0=eye(L) @
const2 = 1/sqrt(tend);   @ Scaing Multiplier when W0 is not eye(L) @
                    @ These scaling multipliers should be set so that
                         the value of function (vof) in nonlinear search of
                         MINQUAD.SET is near 1.  If vof is too close to 0,
                         the search will not work properly.

                         const2=1/sqrt(tend) will generally be good. @
w0flag=0;    @ scalar;
                If w0flag=0, W0=I is used as the initial weighing matrix W0.

                If w0flag=1, initial bgm is used to calculate initial W0.

                If w0flag=2, W0 in the memory is used as initial W0.

                If w0flag=3, W0 and bgm in the memory are used to give
                       the first GMM result.                       @

maxitegm=2; @ Sets maximum # of iteration over weighting matrix, W0.  Set
                 w0flag=0 and maxitegm=2 to execute usual 2-Stage GMM. @

zero=1E-9;    @ Iteration over W0 continues until the maximum
                 difference of the current and the previous W0 in
                 absolute value becomes less than 'zero', or
                 the # of iteration exceeds maxitegm. @

calwflag=4;  @ This variable is used to choose the method to calculate
               the distance matrics, W0.

               If calwflag=0, Durbin's method will be used when W0
                   is singular.

               If calwflag=1, Newey and West Method will be used when
                   W0 is singular.

               If calwflag=2, Parzen's lag window will be used when
                  W0 is singular.

               If calwflag=3, Durbin's method will be used.

               If calwflag=4, Newey and West Method will be used.

               If calwflag=5, Parzen's lag window will be used.

           Durbin's method imposes zero restrictions while Newey and West
           method dose not.  @

ordard=floor(sqrt(8190/sumc(nzv)^2)); @ Order of AR representation for
                 Durbin's method when W0 is singular @

lend=25;   @ Order of lags used for Newey and West method @

    @ See MINQUAD.SET for the following globals @
hflag=2;
dfpflag=1;
sstol=1e-39;
    @ See MAXMUM.DOC on MODULE9 of GAUSS for the following globals @
gradtol=8e-5;
btol=1e-5;
typf=1;
typb=1;

@ *************************************************************** @
@ ----------- Step 3: READING IN DATA --------------------------- @
@ *************************************************************** @

t=tend+1;


load xdat[240,5] = c:\dat\exchange;
 xdat= xdat[6:rows(xdat),.];
@ x=100*ln(xdat[.,4]); @  @ German exchange rate @
@ x=100*ln(xdat[.,2]); @  @ British exchange rate @
@ x=100*ln(xdat[.,5]);  @  @Japanese exchange rate @
@ x=100*ln(xdat[.,4]./xdat[.,2]);@ @ marks per pound @
 x=100*ln(xdat[.,5]./xdat[.,4]); @ yen per mark @
@ x=100*ln(xdat[.,5]./xdat[.,2]); @ @ yen per pound@

load ydat[239,6] = c:\dat\cpi;
  ydat=ydat[6:rows(ydat),.];
k=2; do while k <= 6;
 ydat[.,k]=100*ydat[.,k]/ydat[170,k]; k=k+1; endo;
@ y=100*ln(ydat[.,2]./ydat[.,4]); @    @ U.S.-German price ratio @
@  y=100*ln(ydat[.,2]./ydat[.,3]);@  @ US-UK price ratio @
@ y=100*ln(ydat[.,2]./ydat[.,5]);@  @US-Japan price ratio @
@ y=100*ln(ydat[.,3]./ydat[.,4]);@ @British-German price ratio@
 y1= 100*ln(ydat[.,4]); y2=100*ln(ydat[.,5]);
 y=100*ln(ydat[.,4]./ydat[.,5]); @German-Japanese price ratio@
@ y=100*ln(ydat[.,3]./ydat[.,5]);@ @British-Japanese price ratio@

yx= x[1:234];
ydif=y[2:234]-y[1:233];
xdif=x[2:234]-x[1:233];
yxdif=yx[2:234]-yx[1:233];
y2dif=y2[2:234]-y2[1:233];
y1dif=y1[2:234]-y1[1:233];

ydif=y[2:234]-y[1:233];
xdif=x[2:234]-x[1:233];
xmat=ones(rows(y),1)~y1~y2;
fit= xmat*invpd(xmat'xmat)*xmat'yx;
betols= invpd(xmat'xmat)*xmat'yx;
  betols;

ny=rows(y);
psi= (yx[2:rows(yx)]-fit[2:rows(fit)])~(y1dif-sumc(y1dif)/rows(y1dif))~
       (y2dif-sumc(y2dif)/rows(y2dif));
u1=psi[.,1]; v1=psi[.,2]; v2=psi[.,3];
cov=psi'psi/rows(v2);

@ form long-run covariance matrix of psi following Pagan @
ut=u1[2:rows(u1)]; utlag=u1[1:rows(u1)-1];
vt=v1[2:rows(v2)]; vtlag=v1[1:rows(v2)-1];
vtt= v2[2:rows(v2)]; vttlag=v2[1:rows(v2)-1];
xx=utlag~vtlag~vttlag;
row1= invpd(xx'xx)*(xx'ut);
row2= invpd(xx'xx)*(xx'vt);
row3= invpd(xx'xx)*(xx'vtt);
res1= (ut-xx*row1)~(vt-xx*row2)~(vtt-xx*row3);

@ form autocovs of res with itself lagged @
n=rows(res1);
k=40; j=2; lam=zeros(3,3); do while j <= k;
lam= lam+ (1-(j-1)/(k+1))*res1[j:n,.]'res1[1:n-j+1,.]/(n-j+1);
j=j+1;
endo;

rhoo= row1'|row2'|row3';
rho1= inv(eye(3)-rhoo);
lamu= rho1*cov'rhoo + rho1*lam*rho1;

 @ form omega matrix, long run varaince matrix. MA formula used @

omega=cov+lamu+lamu';

@ from first correction to betas result is bet1 @

meg22inv=inv(omega[2:3,2:3]);
cor1=omega[1,2:3]*meg22inv;
a= cov[1,2:3]+lamu[2:3,1]' - cor1*(cov[2:3,2:3]+lamu[2:3,2:3]);
a=a';
@first correction @
x2mat= xmat[2:rows(xmat),.];
bet=invpd(x2mat'x2mat)*x2mat'(yx[2:rows(yx)]-psi[.,2:3]*cor1');
 bet;
x2mat= zeros(rows(xmat),2);
k=2; do while k <= 3;
 x2mat[.,k-1]=xmat[.,k]-sumc(xmat[.,k])/rows(xmat[.,k]);
 k=k+1; endo;
@ second correction to fully modified estimator @
bet[2:3]= bet[2:3]-invpd(x2mat'x2mat)*a;
bet[1]= sumc(yx-xmat[.,2:3]*bet[2:3])/rows(yx);
  bet;
 bet=betols;
     /*
  bet represents the starting values obtained from Phillip-Hansen estimates
  of cointegrating vectors.  Phillips.prg is the program used to obtain these
  estimates for each particular set of countries.  This version of the program
  uses estimates for Germany-Japan.  However, different starting values can be 
  used by simply uncommenting those values for the set of countries that you
  are interested in and commenting out the values for Germany-Japan.
     */




@ bet= .5396|-1.446|1.2168;@  @us-uk @
@ bet= -9.964|-3.346|5.69;@  @US-GER@
@ bet= 6.585|-1.296|.9962;@  @US-JAPAN@
 bet= 8.898|-2.021|1.051;  @GER-JAPAN@
@ bet= 5.848|-1.159|1.134; @ @UK-JAPAN@
@ bet= -.3265|-.903|1.244; @ @UK-GER@

yxres= yx - xmat*bet;
yxrdif= yxres[2:rows(yxres)]-yxres[1:rows(yxres)-1];

@ ******************************************************************** @
/* -------------------------------------------------------------------
          Step 4: Define the proc hu(b) that returns tend by L matrix
                             |[z(1)w(1)]'      |
                             |    |            |
                             |[z(tend)w(tend)]'|

                     where [z(t)w(t)]'=[w1(t)z1(t)',...,wnw(t)znw(t)']
----------------------------------------------------------------------- */
@ ********************************************************************** @

@ The proc must have the name "hu", and should take only one argument. @

@ Note that the proc "hu" will be called many times in GMM procedure.
  If some calculations can be done outside the proc, that will speed up
  the program. For example, here vectors of instrumental variables are
  constructed outside the proc. @

  @ Construct instrumental variables @
  @ construct periodogram and sample coherence @

zpy= ydif-sumc(ydif)/rows(ydif);
zpx= xdif-sumc(xdif)/rows(xdif);


zpyx= yxrdif-sumc(yxrdif)/rows(yxrdif);

@ THIS PROGRAM TAPERS THE DATA @
timez= seqa(1,1,tend);
mm= 24;
j=1; do while j <= mm;
 wtt= .5*(1-cos(pi*(timez[j]-.5)/mm));
 zpy[j]= wtt*zpy[j];
 zpx[j]= wtt*zpx[j];
 zpyx[j]= wtt*zpyx[j]; j=j+1; endo;
j= tend-mm; do while j <= tend;
 wtt= .5*(1-cos(pi*(tend-timez[j]+.5)/mm));
 zpy[j]= wtt*zpy[j];
 zpx[j]= wtt*zpx[j];
 zpyx[j]= wtt*zpyx[j]; j=j+1; endo;
 zpx=zpx-sumc(zpx)/rows(zpx);
 zpy=zpy-sumc(zpy)/rows(zpy);
 zpyx=zpyx-sumc(zpyx)/rows(zpyx);

@ bgm[4]= ln(zpyx'zpyx/(rows(zpyx)-1));@

@ an = (1/tend)*(sumc(y.*(-ones(tend,1)).^timez))^2; @
per=zeros(tend,1);
pe= zeros(tend,1);
pec= zeros(tend,1);

lam=zeros(tend,1);
lam= 2*pi*timez[1:tend]/(tend+1);
z1 = complex(cos(lam),sin(lam));
z2 = complex(cos(lam),-sin(lam));

@ periodogram ordinates are in per @
j= 1; do while j <= tend;
 temp = z2[j]*ones(tend,1);
 per[j]= (2/(tend))*abs(sumc(zpy[1:tend].*temp.^timez))^2;
 pe[j]= (2/(tend))*abs(sumc(zpx[1:tend].*temp.^timez))^2;
 pec[j]= (2/(tend))*abs(sumc(zpyx[1:tend].*temp.^timez))^2;
j=j+1; endo;

per=per/(4*pi);
pe = pe/(4*pi);
pec = pec/(4*pi);

tr=10; hmax=1;
j=1; do while j <= tr-1;
hmax= hmax + 2*j/tr; j=j+1; endo; hmax= 1/hmax;
coef=zeros(2*tr-1,1);

j=1; do while j <= 2*tr-1;
if j <= tr; coef[j]= (j/tr)*hmax; endif;
if j > tr; coef[j]= (2-j/tr)*hmax; endif;
j=j+1; endo;

per2=per;
pe2=pe;
pec2=pec;
j=tr; do while j <= tend-tr+1;
 per2[j]= per[j-tr+1:j+tr-1]'coef;
 pe2[j]= pe[j-tr+1:j+tr-1]'coef;
 pec2[j]= pec[j-tr+1:j+tr-1]'coef;
 j=j+1; endo;

j= tend-tr+2; do while j <= tend;
 per2[j]= (per[j-tr+1:rows(per)]|per[1:j+tr-rows(per)-1])'coef;
 pe2[j]= (pe[j-tr+1:rows(pe)]|pe[1:j+tr-rows(pe)-1])'coef;
 pec2[j]= (pec[j-tr+1:rows(pec)]|pec[1:j+tr-rows(pec)-1])'coef;
j=j+1; endo;
j=1; do while j <= tr-1;
 per2[j]= per2[tend-j+1];
 pe2[j] = pe2[tend-j+1];
 pec2[j]= pec2[tend-j+1];
j=j+1; endo;

{par,f2,g2,retcode2} = optmum(&hu2,bgm);
par; g2; f2;

f2= hu3(par[1:rows(par)-1]);
cova = 2*f2*inv(hessp(&hu3,par[1:rows(par)-1]))/rows(lam);
ster = sqrt(diag(cova)); ster;
cova;


PROC (1) = hu2(b);
 local d,phi1,the1,spd,dphat1;
b=real(b);
d=b[1];
  phi1= z1*b[2]+z1^2*b[3]+1;
  the1= z1*b[4]+1;
  spd = exp(b[5])*(1-z1)^(-d).*(1-z2)^(-d).*(abs(the1)^2)./(abs(phi1)^2);
  dphat1= ln(real(spd)) + real(pe)./(real(spd));
retp(sumc(dphat1));
endp;

PROC (1) = hu3(b);
   local d,phi1,the1,spd,dphat1;
d=b[1];
  phi1= z1*b[2]+z1^2*b[3]+1;
  the1= z1*b[4]+1;
  spd = (1-z1)^(-d).*(1-z2)^(-d).*(abs(the1)^2)./(abs(phi1)^2);
  dphat1 = real(pe)./(real(spd));
retp(sumc(dphat1));
endp;

PROC (1) = hu(b);
 local d,phi1,the1,spd,dphat1,dd,dd2,dp,dphi,zw,dhess;
  dp=zeros(rows(lam),4);
b=real(b);
d=b[1];
  phi1= z1*b[2]+z1^2*b[3]+1;
  the1= z1*b[4]+1;
  dd2 = (1-z1)^(-d).*(1-z2)^(-d).*(abs(the1)^2)./(abs(phi1)^2);
  spd = exp(b[5])*dd2;
  dphat1=  1/real(dd2) - real(pe)./(exp(b[5])*real(dd2)^2);
  dd= (1-z1)^(-d).*(1-z2)^(-d);
  dp[.,1]= -1*ln(-2*cos(lam)+2).*(-2*cos(lam)+2)^(-1*d);
  dp[.,1]= dp[.,1].*(abs(the1)^2)./(abs(phi1)^2);
  dphi= -1/(abs(phi1)^2)^2;
  dp[.,2]= (2*real(phi1).*real(z1)+2*imag(phi1).*imag(z1))
           .*dd.*dphi.*abs(the1)^2;
 dp[.,3]= (2*real(phi1).*real(z1^2)+2*imag(phi1).*imag(z1^2))
           .*dd.*dphi.*abs(the1)^2;
 dp[.,4]= (2*real(the1).*real(z1)+2*imag(the1).*imag(z1))
           .*dd./abs(phi1)^2;


  dhess= ones(rows(dd2),1) - real(pe)./(exp(b[5])*real(dd2));
  zw= (dp[.,1:4].*dphat1)~(dhess);
retp(zw);
endp;

@=========== The User does not have to change the code below. ============== @

@--------------------- Print results of estimation---------------------@
proc (0)=prntrslt(x,s);

? "
--------------- Minimization Results ----------------------
Step size: " s "
Value of the objective function: " vof "
Minimiser is"  x' "
-----------------------------------------------------------";

endp;

@ ------------------------------------------------------------------------- @
@ ************************************************************************ @
@ THE PROGRAM STARTS @
bgm=par;
call gmmq(gradname,tend,nzv,rwv,nw,rows(bgm),zero,maxitegm);

output off;
end;
@ ------------------------ END OF PROGRAM --------------------------------- @





















